@@ -4,6 +4,7 @@
  * LICENSE-APACHE for a copy of the license.
  */
 #include <postgres.h>
+#include <access/amapi.h>
 #include <fmgr.h>
 #include <utils/lsyscache.h>
 #include <utils/timestamp.h>
@@ -78,6 +79,8 @@ CROSSMODULE_WRAPPER(array_compressor_finish);
 CROSSMODULE_WRAPPER(create_compressed_chunk);
 CROSSMODULE_WRAPPER(compress_chunk);
 CROSSMODULE_WRAPPER(decompress_chunk);
+CROSSMODULE_WRAPPER(hypercore_handler);
+CROSSMODULE_WRAPPER(hypercore_proxy_handler);
 
 /* continuous aggregate */
 CROSSMODULE_WRAPPER(continuous_agg_invalidation_trigger);
@@ -95,6 +98,10 @@ CROSSMODULE_WRAPPER(chunk_create_empty_table);
 
 CROSSMODULE_WRAPPER(recompress_chunk_segmentwise);
 CROSSMODULE_WRAPPER(get_compressed_chunk_index_for_recompression);
+CROSSMODULE_WRAPPER(merge_chunks);
+
+/* hypercore */
+CROSSMODULE_WRAPPER(is_compressed_tid);
 
 /*
  * casting a function pointer to a pointer of another type is undefined
@@ -113,6 +120,32 @@ error_no_default_fn_community(void)
 					 "Timescale Cloud.")));
 }
 
+static bytea *
+error_hypercore_proxy_index_options(Datum reloptions, bool validate)
+{
+	error_no_default_fn_community();
+	return NULL;
+}
+
+/*
+ * An index AM always needs to return a IndexAmRoutine because the handler
+ * function is invoked when the default opclass for a type is defined in
+ * SQL. Therefore, return this dummy under non-TSL license and error out when
+ * parsing index options instead.
+ */
+static Datum
+error_pg_community_hypercore_proxy_handler(PG_FUNCTION_ARGS)
+{
+	IndexAmRoutine *amroutine = makeNode(IndexAmRoutine);
+
+	amroutine->amstrategies = 0;
+	amroutine->amsupport = 1;
+	amroutine->amoptsprocnum = 0;
+	amroutine->amoptions = error_hypercore_proxy_index_options;
+
+	PG_RETURN_POINTER(amroutine);
+}
+
 static bool
 error_no_default_fn_bool_void_community(void)
 {
@@ -261,7 +294,7 @@ ts_tsl_loaded(PG_FUNCTION_ARGS)
 }
 
 static void
-preprocess_query_tsl_default_fn_community(Query *parse)
+preprocess_query_tsl_default_fn_community(Query *parse, int *cursor_opts)
 {
 	/* No op in community licensed code */
 }
@@ -276,8 +309,11 @@ TSDLLEXPORT CrossModuleFunctions ts_cm_functions_default = {
 	.set_rel_pathlist_dml = NULL,
 	.set_rel_pathlist_query = NULL,
 	.set_rel_pathlist = NULL,
+	.ddl_command_start = NULL,
+	.ddl_command_end = NULL,
 	.process_altertable_cmd = NULL,
 	.process_rename_cmd = NULL,
+	.process_explain_def = NULL,
 
 	/* gapfill */
 	.gapfill_marker = error_no_default_fn_pg_community,
@@ -360,6 +396,9 @@ TSDLLEXPORT CrossModuleFunctions ts_cm_functions_default = {
 	.dictionary_compressor_finish = error_no_default_fn_pg_community,
 	.array_compressor_append = error_no_default_fn_pg_community,
 	.array_compressor_finish = error_no_default_fn_pg_community,
+	.hypercore_handler = error_no_default_fn_pg_community,
+	.hypercore_proxy_handler = error_pg_community_hypercore_proxy_handler,
+	.is_compressed_tid = error_no_default_fn_pg_community,
 
 	.show_chunk = error_no_default_fn_pg_community,
 	.create_chunk = error_no_default_fn_pg_community,
@@ -369,6 +408,7 @@ TSDLLEXPORT CrossModuleFunctions ts_cm_functions_default = {
 	.recompress_chunk_segmentwise = error_no_default_fn_pg_community,
 	.get_compressed_chunk_index_for_recompression = error_no_default_fn_pg_community,
 	.preprocess_query_tsl = preprocess_query_tsl_default_fn_community,
+	.merge_chunks = error_no_default_fn_pg_community,
 };
 
 TSDLLEXPORT CrossModuleFunctions *ts_cm_functions = &ts_cm_functions_default;
